set ns [new Simulator]

# There are 3 sources each generating a UDP flow, they share a bottleneck link and a single destination. Their number is given by the parameter NodeNb

set selecSched 1;
set weightWRR1 10;
set weightWRR2 5;
set weightWRR3 2;

set cir1 10Mb; # policing parameter
set rate1 10Mb;
set cir2 5Mb; # policing parameter
set rate2 10Mb;
set cir3 5Mb; # policing parameter
set rate3 10Mb;
set cbs1 100; # policing parameter
set packetSize 1000;
set NodeNb 3; # number of source nodes
set NumberFlows 1; # number of flows per source node
set Duration 50; # duration of simulation

set tf [open out.tr w]; # Open trace file
$ns trace-all $tf

# Open the NAM trace file
set tfnam [open out.nam w];
#$ns namtrace-all $tfnam

set param [open nodemap w]
#Define a 'finish' procedure
proc finish {} {
	global ns tf tfnam param
	$ns flush-trace
	close $tf
	close $tfnam
	close $param
#	exec nam out.nam &
	exit 0
}

# Define the topology
set D [$ns node]
set Ed [$ns node]
set Eg [$ns node]
set Core [$ns node]

$ns simplex-link $Ed $Core 10Mb 10ms dsRED/edge
$ns simplex-link $Core $Ed 10Mb 10ms dsRED/core
$ns duplex-link $Ed $D 20Mb 1ms DropTail
$ns simplex-link $Eg $Core 10Mb 10ms dsRED/edge
$ns simplex-link $Core $Eg 10Mb 10ms dsRED/core
$ns queue-limit $Core $Ed 100
$ns queue-limit $Core $Eg 100

for {set j 1} {$j<=$NodeNb} {incr j} {
	set S($j) [$ns node]
	$ns duplex-link $S($j) $Eg 7Mb 1ms DropTail
	$ns queue-limit $S($j) $Eg 100
}


################# Eg to Core

set qEgC [[$ns link $Eg $Core] queue]
$qEgC meanPktSize $packetSize
$qEgC set numQueues_ 3
$qEgC set NumPrec 2

########## Shaping and policing of the flow
$qEgC addPolicyEntry [$S(1) id] [$D id] TokenBucket 001 $cir1 $cbs1
$qEgC addPolicyEntry [$S(2) id] [$D id] TSW2CM 011 $cir2
$qEgC addPolicyEntry [$S(3) id] [$D id] TSW2CM 101 $cir3

$qEgC addPolicerEntry TokenBucket 001 010
$qEgC addPolicerEntry TSW2CM 011 100
$qEgC addPolicerEntry TSW2CM 101 110

########## Connecting the policing to the queue management

$qEgC addPHBEntry 001 0 0
$qEgC addPHBEntry 010 0 1
$qEgC addPHBEntry 011 1 0
$qEgC addPHBEntry 100 1 1
$qEgC addPHBEntry 101 2 0
$qEgC addPHBEntry 110 2 1

$qEgC setMREDMode WRED 0
$qEgC setMREDMode WRED 1
$qEgC setMREDMode WRED 2
$qEgC configQ 0 0 20 30 0.1
$qEgC configQ 0 1 10 15 0.1
$qEgC configQ 1 0 20 30 0.1
$qEgC configQ 1 1 10 15 0.1
$qEgC configQ 2 0 20 30 0.1
$qEgC configQ 2 1 10 15 0.1
if {$selecSched==0} {
	$qEgC setSchedularMode PRI
} else {
	$qEgC setSchedularMode WRR
	$qEgC addQueueWeights 0 $weightWRR1
	$qEgC addQueueWeights 1 $weightWRR2
	$qEgC addQueueWeights 2 $weightWRR3
}



################# Core to Eg

set qCEg [[$ns link $Core $Eg] queue]
$qCEg meanPktSize 40
$qCEg set numQueues_ 3
$qCEg set NumPrec 2

########## Connecting the policing to the queue management

$qCEg addPHBEntry 001 0 0
$qCEg addPHBEntry 010 0 1
$qCEg addPHBEntry 011 1 0
$qCEg addPHBEntry 100 1 1
$qCEg addPHBEntry 101 2 0
$qCEg addPHBEntry 110 2 1
$qCEg setMREDMode WRED 0
$qCEg setMREDMode WRED 1
$qCEg setMREDMode WRED 2
$qCEg configQ 0 0 20 30 0.1
$qCEg configQ 0 1 10 15 0.1
$qCEg configQ 1 0 20 30 0.1
$qCEg configQ 1 1 10 15 0.1
$qCEg configQ 2 0 20 30 0.1
$qCEg configQ 2 1 10 15 0.1
if {$selecSched==0} {
	$qCEg setSchedularMode PRI
} else {
	$qCEg setSchedularMode WRR
	$qCEg addQueueWeights 0 $weightWRR1
	$qCEg addQueueWeights 1 $weightWRR2
	$qCEg addQueueWeights 2 $weightWRR3
}

################## Ed to Core
set qEdC [[$ns link $Ed $Core] queue]
$qEdC meanPktSize 40
$qEdC set numQueues_ 3
$qEdC set NumPrec 2

########## Shaping and policing of the flow
$qEdC addPolicyEntry [$D id] [$S(1) id] TokenBucket 001 $cir1 $cbs1
$qEdC addPolicyEntry [$D id] [$S(2) id] TSW2CM 011 $cir2
$qEdC addPolicyEntry [$D id] [$S(3) id] TSW2CM 101 $cir3

$qEdC addPolicerEntry TokenBucket 001 010
$qEdC addPolicerEntry TSW2CM 011 100
$qEdC addPolicerEntry TSW2CM 101 110

########## Connecting the policing to the queue management

$qEdC addPHBEntry 001 0 0
$qEdC addPHBEntry 010 0 1
$qEdC addPHBEntry 011 1 0
$qEdC addPHBEntry 100 1 1
$qEdC addPHBEntry 101 2 0
$qEdC addPHBEntry 110 2 1

$qEdC setMREDMode WRED 0
$qEdC setMREDMode WRED 1
$qEdC setMREDMode WRED 2
$qEdC configQ 0 0 20 30 0.1
$qEdC configQ 0 1 10 15 0.1
$qEdC configQ 1 0 20 30 0.1
$qEdC configQ 1 1 10 15 0.1
$qEdC configQ 2 0 20 30 0.1
$qEdC configQ 2 1 10 15 0.1
if {$selecSched==0} {
	$qEdC setSchedularMode PRI
} else {
	$qEdC setSchedularMode WRR
	$qEdC addQueueWeights 0 $weightWRR1
	$qEdC addQueueWeights 1 $weightWRR2
	$qEdC addQueueWeights 2 $weightWRR3
}



#############Printing########################
$qEdC printPolicyTable
$qEdC printPolicerTable

################# Core to Ed
set qCEd [[$ns link $Core $Ed] queue]
$qCEd meanPktSize $packetSize
$qCEd set numQueues_ 3
$qCEd set NumPrec 2

########## Connecting the policing to the queue management
$qCEd set numQueues_ 3
$qCEd set NumPrec 2
$qCEd addPHBEntry 001 0 0
$qCEd addPHBEntry 010 0 1
$qCEd addPHBEntry 011 1 0
$qCEd addPHBEntry 100 1 1
$qCEd addPHBEntry 101 2 0
$qCEd addPHBEntry 110 2 1
$qCEd setMREDMode WRED 0
$qCEd setMREDMode WRED 1
$qCEd setMREDMode WRED 2
$qCEd configQ 0 0 20 30 0.1
$qCEd configQ 0 1 10 15 0.1
$qCEd configQ 1 0 20 30 0.1
$qCEd configQ 1 1 10 15 0.1
$qCEd configQ 2 0 20 30 0.1
$qCEd configQ 2 1 10 15 0.1
if {$selecSched==0} {
	$qCEd setSchedularMode PRI
} else {
	$qCEd setSchedularMode WRR
	$qCEd addQueueWeights 0 $weightWRR1
	$qCEd addQueueWeights 1 $weightWRR2
	$qCEd addQueueWeights 2 $weightWRR3
}


#Create a random generator for starting the ftp and for bottleneck link delays
#set rng [new RNG]
#$rng seed 2

#Parameters for RV for starting times of ftp connections
#set RVstart [new RandomVariable/Uniform]
#$RVstart set min_ 0
#$RVstart set max_ 7
#$RVstart use-rng $rng

# UDP and TCP sources, dest and connections

#Setup a UDP connection from S(1)
set udp(1) [new Agent/UDP]
$ns attach-agent $S(1) $udp(1)
set null(1) [new Agent/Null]
$ns attach-agent $D $null(1)
$ns connect $udp(1) $null(1)
$udp(1) set fid_ 1
#Setup a CBR over UDP connection from S(1)
set cbr(1) [new Application/Traffic/CBR]
$cbr(1) attach-agent $udp(1)
$cbr(1) set type_ CBR
$cbr(1) set packet_size_ $packetSize
$cbr(1) set rate_ $rate1
$cbr(1) set random_ 1
$ns at 0 "$cbr(1) start"
$ns at [expr 0+30] "$cbr(1) stop"

#Setup a UDP connection from S(2)
set udp(2) [new Agent/UDP]
$ns attach-agent $S(2) $udp(2)
set null(2) [new Agent/Null]
$ns attach-agent $D $null(2)
$ns connect $udp(2) $null(2)
$udp(2) set fid_ 2
#Setup a CBR over UDP connection from S(2)
set cbr(2) [new Application/Traffic/CBR]
$cbr(2) attach-agent $udp(2)
$cbr(2) set type_ CBR
$cbr(2) set packet_size_ $packetSize
$cbr(2) set rate_ $rate2
$cbr(2) set random_ 1
$ns at 0 "$cbr(2) start"
$ns at [expr 0+$Duration] "$cbr(2) stop"

#Setup a UDP connection from S(3)
set udp(3) [new Agent/UDP]
$ns attach-agent $S(3) $udp(3)
set null(3) [new Agent/Null]
$ns attach-agent $D $null(3)
$ns connect $udp(3) $null(3)
$udp(3) set fid_ 3
#Setup a CBR over UDP connection from S(3)
set cbr(3) [new Application/Traffic/CBR]
$cbr(3) attach-agent $udp(3)
$cbr(3) set type_ CBR
$cbr(3) set packet_size_ $packetSize
$cbr(3) set rate_ $rate3
$cbr(3) set random_ 1
$ns at 0 "$cbr(3) start"
$ns at [expr 0+$Duration] "$cbr(3) stop"

for {set i 1} {$i<=$NodeNb} {incr i} {
	puts $param "S($i) [$S($i) id]"
}
puts $param "D [$D id]"
puts $param "Ed [$Ed id]"
puts $param "C [$Core id]"
puts $param "Eg [$Eg id]"
#puts $param "start3 $startT(3,1)"

$ns at [expr $Duration] "finish"
$ns run
