set ns [new Simulator]

set nf [open out.nam w]
$ns namtrace-all $nf

set tf [open out.tr w]
set windowVsTime [open win w]
set param [open parameters w]
$ns trace-all $tf
set queuesize [open QueueSize w]

# Define a 'finish' procedure
proc finish {} {
	global ns nf tf
	$ns flush-trace
	close $nf
	close $tf
#	exec nam out.nam &
#	exec grep "a" red-queue.tr > ave.tr
#	exec grep "Q" red-queue.tr > cur.tr
	exit 0
}


# Create bottleneck and dest nodes
set n2 [$ns node]
set n3 [$ns node]

# Create links between these nodes
$ns duplex-link $n2 $n3 3Mb 20ms DropTail #RED

set NbSrc 3
set Duration 50

# Source nodes
for {set j 1} {$j<=$NbSrc} {incr j} {
	set S($j) [$ns node]
}

# Create a randm generator for starting the ftp and for bottleneck link delays
set rng [new RNG]
$rng seed 2

# Parameters for RV for starting times of ftp connections
set RVstart [new RandomVariable/Uniform]
$RVstart set min_ 0
$RVstart set max_ 7
$RVstart use-rng $rng
for {set i 1} {$i<=$NbSrc} {incr i} {
	set startT($i) [expr [$RVstart value]]
	set dly($i) 1
	puts $param "startT($i) $startT($i) sec"
}

# Links between source and bottleneck
for {set i 1} {$i<=$NbSrc} {incr i} {
	$ns duplex-link $S($i) $n2 10Mb $dly($i)ms DropTail
	$ns queue-limit $S($i) $n2 20
} 

$ns queue-limit $n2 $n3 100

for {set i 1} {$i<=$NbSrc} {incr i} {
	set tcpsrc($i) [new Agent/TCP/Reno]
	$tcpsrc($i) set window_ 8000
	set tcpsnk($i) [new Agent/TCPSink]
}

for {set i 1} {$i<=$NbSrc} {incr i} {
	$ns attach-agent $S($i) $tcpsrc($i)
	$ns attach-agent $n3 $tcpsnk($i)
	$ns connect $tcpsrc($i) $tcpsnk($i)
	set ftp($i) [$tcpsrc($i) attach-source FTP]
	$tcpsrc($i) set packetSize_ 552
	$ns at $startT($i) "$ftp($i) start"
	$ns at $Duration "$ftp($i) stop"
} 

proc plotWindow {tcpSource file k} {
	global ns NbSrc
	set time 0.03
	set now [$ns now]
	set cwnd [$tcpSource set cwnd_]
	if {$k==1} { 
		puts -nonewline $file "$now $cwnd "
	} elseif {$k==$NbSrc} {
		puts $file "$cwnd"
	} else {
		puts -nonewline $file "$cwnd "
	}
	$ns at [expr $now+$time] "plotWindow $tcpSource $file $k"
}

for {set i 1} {$i<=$NbSrc} {incr i} {
	$ns at 0.1 "plotWindow $tcpsrc($i) $windowVsTime $i"
}


set monitor [$ns monitor-queue $n2 $n3 stdout 0.05]

proc queueLength {sum number file} {
	global ns monitor
	set time 0.05
	set len [$monitor set pkts_]
	set now [$ns now]
	puts $file "$now $len"
	$ns at [expr $now+$time] "queueLength $sum $number $file"
}
$ns at 0.1 "queueLength 0 0 $queuesize"

$ns at [expr $Duration] "finish"
$ns run
