#Create the simulator instance
set ns [new Simulator]

#Open the NAM trace file
set nf [open out.nam w]
$ns namtrace-all $nf

set tf [open out.tr w]
set windowVsTime [open win w]
set param [open parameters w]
set queuesize [open QueueSize w]
$ns trace-all $tf

#Define a 'finish' procedure
proc finish {} {
	global ns nf tf
	$ns flush-trace
	close $nf
	close $tf
#	exec nam out.nam &
	exit 0
}

set NbSrc 2
set Duration 200

#Source nodes
for {set j 1} {$j<=$NbSrc} {incr j} {
	set S($j) [$ns node]
}

#Create bottleneck and dest nodes and links
set n2 [$ns node]
set n3 [$ns node]
$ns duplex-link $n2 $n3 0.7Mb 20ms DropTail

#Create a random generator for starting the ftp and for bottleneck link delays
set rng [new RNG]
$rng seed 0

#Parameters for RV for delays
set RVdly [new RandomVariable/Uniform]
$RVdly set min_ 1
$RVdly set max_ 5
$RVdly use-rng $rng

#Parameters for RV for starting times of ftp connections
set RVstart [new RandomVariable/Uniform]
$RVstart set min_ 0
$RVstart set max_ 7
$RVstart use-rng $rng

#We define 2 random parameters for each connetion
for {set i 1} {$i<=$NbSrc} {incr i} {
#	set startT($i) [expr [$RVstart value]]
#	set dly($i) [expr [$RVdly value]]
	set startT($i) [expr ($i-1)*50]
#	set dly($i) 10ms
#	puts $param "dly($i) $dly($i) ms"
	puts $param "startT($i) $startT($i) sec"
}
set dly(1) 100ms
set dly(2) 100ms

#Links between source and bottleneck
for {set i 1} {$i<=$NbSrc} {incr i} {
	$ns duplex-link $S($i) $n2 10Mb $dly($i)ms DropTail
	$ns queue-limit $S($i) $n2 100
}

#Monitor the queue for link n2-n3 (for nam)
$ns duplex-link-op $n2 $n3 orient right
$ns duplex-link-op $n2 $n3 queuePos 0.5
$ns color 1 Blue
$ns color 2 Red

#Set queue size of link n2-n3 to 10
$ns queue-limit $n2 $n3 10

#TCP sources
for {set i 1} {$i<=$NbSrc} {incr i} {
	if {$i==1} {	
		set tcp_src($i) [new Agent/TCP]
	} else {	
		set tcp_src($i) [new Agent/TCP]}
	$tcp_src($i) set fid_ $i
	$tcp_src($i) set packetSize_ 552
	$tcp_src($i) set window_ 100000000
}

#TCP destinations
for {set i 1} {$i<=$NbSrc} {incr i} {
	set tcp_snk($i) [new Agent/TCPSink]
}

#Connections
for {set i 1} {$i<=$NbSrc} {incr i} {
	$ns attach-agent $S($i) $tcp_src($i)
	$ns attach-agent $n3 $tcp_snk($i)
	$ns connect $tcp_src($i) $tcp_snk($i)
}

#FTP sources
for {set i 1} {$i<=$NbSrc} {incr i} {
	set ftp($i) [$tcp_src($i) attach-source FTP]
}

#Parameterization of TCP sources
for {set i 1} {$i<=$NbSrc} {incr i} {
	$tcp_snk($i) set packetSize_ 552
}

#Schedule events for the FTP agents
for {set i 1} {$i<=$NbSrc} {incr i} {
	$ns at $startT($i) "$ftp($i) start"
	$ns at [expr $Duration-($NbSrc-$i)*50] "$ftp($i) stop"
}

proc plotWindow {tcpSource file k} {
	global ns NbSrc
	set time 0.03
	set now [$ns now]
	set cwnd [$tcpSource set cwnd_]
	if {$k==1} { 
		puts -nonewline $file "$now $cwnd "
	} elseif {$k==$NbSrc} {
		puts $file "$cwnd"
	} else {
		puts -nonewline $file "$cwnd "
	}
	$ns at [expr $now+$time] "plotWindow $tcpSource $file $k"
}

for {set i 1} {$i<=$NbSrc} {incr i} {
	$ns at 0.1 "plotWindow $tcp_src($i) $windowVsTime $i"
}

set monitor [$ns monitor-queue $n2 $n3 stdout 0.1]

proc queueLength {sum number file} {
	global ns monitor
	set time 0.1
	set len [$monitor set pkts_]
	set now [$ns now]
	puts $file "$now $len"
	$ns at [expr $now+$time] "queueLength $sum $number $file"
}
$ns at 0.1 "queueLength 0 0 $queuesize"

$ns at [expr $Duration] "finish"
$ns run
